@@ -80,8 +80,6 @@ setGeneric('nlf',function(object,...)standardGeneric("nlf"))
 ## iterated filtering
 setGeneric('mif',function(object,...)standardGeneric("mif"))
 setGeneric("mif2",function(object,...)standardGeneric("mif2"))
-## generate new particles
-setGeneric('particles',function(object,...)standardGeneric("particles"))
 
 ## synthetic likelihood
 setGeneric("probe",function(object,probes,...)standardGeneric("probe"))
