@@ -4,7 +4,7 @@ rw.sd <- function (...) {
     as.list(match.call())[-1L]
 }
 
-pkern.sd <- function (rw.sd, time, paramnames) {
+pkern.sd <- function (rw.sd, time, paramnames, enclos) {
     if (!all(names(rw.sd) %in% paramnames)) {
         unrec <- names(rw.sd)[!names(rw.sd) %in% paramnames]
         stop(sQuote("mif2")," error: the following parameter(s), ",
@@ -15,7 +15,7 @@ pkern.sd <- function (rw.sd, time, paramnames) {
     ivp <- function (sd, lag = 1L) {
         sd*(seq_along(time)==lag)
     }
-    sds <- lapply(rw.sd,eval,envir=list(time=time,ivp=ivp))
+    sds <- lapply(rw.sd,eval,envir=list(time=time,ivp=ivp),enclos=enclos)
     for (n in names(sds)) {
         len <- length(sds[[n]])
         if (len==1) {
@@ -47,16 +47,20 @@ setClass(
 mif2.pfilter <- function (object, params, Np,
                           mifiter, rw.sd, cooling.fn,
                           tol = 1e-17, max.fail = Inf,
-                          transform, verbose, filter.mean,
+                          transform, verbose,
+                          .indices = integer(0),
                           .getnativesymbolinfo = TRUE) {
 
     gnsi <- as.logical(.getnativesymbolinfo)
     transform <- as.logical(transform)
     verbose <- as.logical(verbose)
-    filter.mean <- as.logical(filter.mean)
     mifiter <- as.integer(mifiter)
     Np <- as.integer(Np)
 
+    do_ta <- length(.indices)>0L
+    if (do_ta && length(.indices)!=Np[1L])
+        stop(".indices has improper length",call.=TRUE)
+    
     times <- time(object,t0=TRUE)
     ntimes <- length(times)-1
 
@@ -68,7 +72,7 @@ mif2.pfilter <- function (object, params, Np,
 
         ## perturb parameters
         pmag <- cooling.fn(nt,mifiter)$alpha*rw.sd[,nt]
-        params <- .Call(mif2_computations,params,pmag)
+        .Call(randwalk_perturbation,params,pmag) # NB: 'params' is modified!
 
         if (transform)
             tparams <- partrans(object,params,dir="fromEstimationScale",
@@ -77,12 +81,6 @@ mif2.pfilter <- function (object, params, Np,
         if (nt == 1L) {
             ## get initial states
             x <- init.state(object,params=if (transform) tparams else params)
-
-            if (filter.mean)
-                filt.m <- array(dim=c(nrow(x),ntimes),
-                                dimnames=list(rownames(x),NULL))
-            else
-                filt.m <- array(dim=c(0,0))
         }
 
         ## advance the state variables according to the process model
@@ -134,12 +132,11 @@ mif2.pfilter <- function (object, params, Np,
                 x=X,
                 params=params,
                 Np=Np[nt+1],
-                rw=FALSE,
                 rw_sd=numeric(0),
                 predmean=FALSE,
                 predvar=FALSE,
-                filtmean=filter.mean,
-                trackancestry=FALSE,
+                filtmean=FALSE,
+                trackancestry=do_ta,
                 onepar=FALSE,
                 weights=weights,
                 tol=tol
@@ -153,11 +150,12 @@ mif2.pfilter <- function (object, params, Np,
         all.fail <- xx$fail
         loglik[nt] <- xx$loglik
         eff.sample.size[nt] <- xx$ess
+        if (do_ta) {
+            .indices <- .indices[xx$ancestry]
+        }
 
         x <- xx$states
         params <- xx$params
-        if (filter.mean)
-            filt.m[,nt] <- xx$fm
 
         if (all.fail) { ## all particles are lost
             nfail <- nfail+1
@@ -183,7 +181,7 @@ mif2.pfilter <- function (object, params, Np,
         paramMatrix=params,
         eff.sample.size=eff.sample.size,
         cond.loglik=loglik,
-        filter.mean=filt.m,
+        indices=.indices,
         Np=Np,
         tol=tol,
         nfail=as.integer(nfail),
@@ -193,13 +191,14 @@ mif2.pfilter <- function (object, params, Np,
 
 mif2.internal <- function (object, Nmif, start, Np, rw.sd, transform = FALSE,
                            cooling.type, cooling.fraction.50,
-                           tol = 1e17, max.fail = Inf, 
+                           tol = 1e17, max.fail = Inf,
                            verbose = FALSE, .ndone = 0L,
-                           .paramMatrix = NULL, 
+                           .indices = integer(0),
+                           .paramMatrix = NULL,
                            .getnativesymbolinfo = TRUE, ...) {
-    
+
     pompLoad(object)
-    
+
     transform <- as.logical(transform)
     verbose <- as.logical(verbose)
     gnsi <- as.logical(.getnativesymbolinfo)
@@ -218,11 +217,11 @@ mif2.internal <- function (object, Nmif, start, Np, rw.sd, transform = FALSE,
             start <- apply(paramMatrix,1L,mean)
         } else if (Nmif > 0) {         # initial call
             paramMatrix <- array(data=start,dim=c(length(start),Np[1L]),
-                                 dimnames=list(names(start),NULL))
+                                 dimnames=list(variable=names(start),rep=NULL))
         } else {                        # no work to do
             paramMatrix <- array(dim=c(0,0))
         }
-    } else { 
+    } else {
         paramMatrix <- .paramMatrix
         start <- apply(paramMatrix,1L,mean)
     }
@@ -231,7 +230,7 @@ mif2.internal <- function (object, Nmif, start, Np, rw.sd, transform = FALSE,
                       dimnames=list(iteration=seq.int(.ndone,.ndone+Nmif),
                                     variable=c('loglik','nfail',names(start))))
     conv.rec[1L,] <- c(NA,NA,start)
-    
+
     object <- as(object,"pomp")
 
     if (transform)
@@ -252,8 +251,8 @@ mif2.internal <- function (object, Nmif, start, Np, rw.sd, transform = FALSE,
                 tol=tol,
                 max.fail=max.fail,
                 verbose=verbose,
-                filter.mean=(n==Nmif),
                 transform=transform,
+                 .indices=.indices,
                 .getnativesymbolinfo=gnsi
             ),
             silent=TRUE
@@ -266,7 +265,8 @@ mif2.internal <- function (object, Nmif, start, Np, rw.sd, transform = FALSE,
         paramMatrix <- pfp@paramMatrix
         conv.rec[n+1,-c(1,2)] <- coef(pfp)
         conv.rec[n,c(1,2)] <- c(pfp@loglik,pfp@nfail)
-
+        .indices <- pfp@indices
+        
         if (verbose) cat("mif2 iteration ",n," of ",Nmif," completed\n")
 
     }
@@ -318,17 +318,17 @@ setMethod(
 
         if (missing(Np)) {
             stop(sQuote("mif2")," error: ",sQuote("Np")," must be specified",call.=FALSE) }
-            else if (is.function(Np)) {
-                Np <- try(
-                    vapply(seq.int(1,ntimes),Np,numeric(1)),
-                    silent=FALSE
-                )
-                if (inherits(Np,"try-error"))
-                    stop(sQuote("mif2")," error: if ",sQuote("Np"),
-                         " is a function, it must return a single positive integer")
-            } else if (!is.numeric(Np))
-                stop(sQuote("mif2")," error: ",sQuote("Np"),
-                     " must be a number, a vector of numbers, or a function")
+        else if (is.function(Np)) {
+            Np <- try(
+                vapply(seq.int(1,ntimes),Np,numeric(1)),
+                silent=FALSE
+            )
+            if (inherits(Np,"try-error"))
+                stop(sQuote("mif2")," error: if ",sQuote("Np"),
+                     " is a function, it must return a single positive integer")
+        } else if (!is.numeric(Np))
+            stop(sQuote("mif2")," error: ",sQuote("Np"),
+                 " must be a number, a vector of numbers, or a function")
         if (length(Np)==1) {
             Np <- rep(Np,times=ntimes)
         } else if (length(Np)>ntimes) {
@@ -343,7 +343,8 @@ setMethod(
         if (missing(rw.sd))
             stop(sQuote("mif2")," error: ",sQuote("rw.sd")," must be specified!",call.=FALSE)
         if (!is.matrix(rw.sd)) {
-            rw.sd <- pkern.sd(rw.sd,time=time(object),paramnames=names(start))
+            rw.sd <- pkern.sd(rw.sd,time=time(object),paramnames=names(start),
+                              enclos=parent.frame())
         }
 
         cooling.type <- match.arg(cooling.type)
@@ -398,7 +399,7 @@ setMethod(
         if (missing(transform)) transform <- object@transform
         if (missing(cooling.type)) cooling.type <- object@cooling.type
         if (missing(cooling.fraction.50)) cooling.fraction.50 <- object@cooling.fraction.50
-        
+
         if (missing(Np)) Np <- object@Np
         if (missing(tol)) tol <- object@tol
 
