@@ -14,8 +14,8 @@ SEXP do_rmeasure (SEXP object, SEXP x, SEXP times, SEXP params, SEXP gnsi)
   pompfunmode mode = undef;
   int ntimes, nvars, npars, ncovars, nreps, nrepsx, nrepsp, nobs;
   SEXP Snames, Pnames, Cnames, Onames;
-  SEXP tvec, xvec, pvec, cvec;
-  SEXP fun, fn, fcall, rho, ans, nm;
+  SEXP cvec, tvec = R_NilValue, xvec = R_NilValue, pvec = R_NilValue;
+  SEXP fn, fcall, rho = R_NilValue, ans, nm;
   SEXP pompfun;
   SEXP Y;
   int *dim;
